#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

#  bundle_checklocalisations.rb
#  
#  Copyright (c) 2012 Realmac Software. All rights reserved.

require 'osx/cocoa'
include OSX

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

def usage
	print "Usage:\n"
	print "-bundlePath: path to a bundle whose lproj directories should be verified [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby bundle_checklocalisations.rb\n -bundlePath ~/Projects/Products/Release\\ -\\ Standard/Courier.app\n"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def start
	print "========= LOCALISATION VALIDATION STARTED ==========\n"
	print "\n"
end

def finish
	print "\n"
	print "========= LOCALISATION VALIDATION SUCCESS ==========\n"
	exit 0
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "========= LOCALISATION VALIDATION FAILURE ==========\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end


# Check arguments

bundlePath = NSUserDefaults.standardUserDefaults.stringForKey "bundlePath"
if bundlePath.nil?
	fail "a bundle path must be specified using -bundlePath"
	exit 1
end
unless NSFileManager.defaultManager.fileExistsAtPath bundlePath
	fail "bundle does not exist at the specified path"
	exit 1
end
bundleURL = NSURL.fileURLWithPath bundlePath
bundle = NSBundle.bundleWithURL bundleURL
if bundle.nil?
	fail "couldn't load bundle"
	exit 1
end

bundleName = bundlePath.lastPathComponent


#

start


#

resourcesFolderContents = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error bundle.resourceURL.URLByResolvingSymlinksInPath, NSArray.array, 0
unless resourcesFolderContents[0]
	fail "couldn't get contents of resources folder #{resourcesFolderContents[1].localizedDescription}", :fatal
end
resourcesFolderContents = resourcesFolderContents[0]


# Check that the bundle doesn't contain localisable files outside of a localisation region folder

print "Checking #{bundleName} doesn't have any localisable files outside of an lproj folder...\n"

localisableFileExtensions = NSMutableArray.array
localisableFileExtensions << "nib"
localisableFileExtensions << "strings"
localisableFileExtensions << "txt"

localisableFilesInRootResources = resourcesFolderContents.pathsMatchingExtensions localisableFileExtensions
unless localisableFilesInRootResources.count == 0
	joinedLocalisableFilesInRootResources = localisableFilesInRootResources.valueForKey_("lastPathComponent").componentsJoinedByString_ "\n"
	fail "found the following localisable files outside an lproj folder\n#{joinedLocalisableFilesInRootResources}"
end


# Get all localisation regions

localisationFolders = resourcesFolderContents.pathsMatchingExtensions NSArray.arrayWithObject("lproj")
unless localisationFolders.count > 0
	print "didn't find any localisation folders to check\n"
	finish
end

def _resourcesFileNamesInBundleAtLocation(bundleLocation)
	resourceFileNames = NSMutableArray.array
	
	bundleDirectoryEnumerator = NSFileManager.defaultManager.enumeratorAtPath bundleLocation.path
	while currentFileName = bundleDirectoryEnumerator.nextObject do
		currentLocation = bundleLocation.URLByAppendingPathComponent currentFileName
		
		getCurrentLocationIsPackage = currentLocation.getResourceValue_forKey_error NSURLIsPackageKey
		unless getCurrentLocationIsPackage[0]
			fail "couldn't determine whether resource #{currentFileName} is a package #{getCurrentLocationIsPackage[2].localizedDescription}"
			next
		end
		currentLocationIsPackage = getCurrentLocationIsPackage[1]
		bundleDirectoryEnumerator.skipDescendants if currentLocationIsPackage.boolValue
		
		resourceFileNames << currentFileName
	end
		
	resourceFileNames
end

# Check the named development region is present

print "Checking #{bundleName} development region localisation exists...\n"
developmentRegionLocation = nil
developmentRegion = bundle.objectForInfoDictionaryKey_ "CFBundleDevelopmentRegion"
unless developmentRegion.nil?
	developmentRegionLocation = bundle.resourceURL.URLByAppendingPathComponent(developmentRegion).URLByAppendingPathExtension("lproj")
	fail "development localisation region folder not present at #{developmentRegionLocation.path}", :fatal unless NSFileManager.defaultManager.fileExistsAtPath developmentRegionLocation.path
end

# Check the Base localisation if present

print "Checking Base localisation exists...\n"
baseLocalisationLocation = bundle.resourceURL.URLByAppendingPathComponent("Base").URLByAppendingPathExtension("lproj")
baseLocalisationLocation = nil unless NSFileManager.defaultManager.fileExistsAtPath baseLocalisationLocation.path

baseLocalisableResourceFileNames = baseLocalisationLocation ? _resourcesFileNamesInBundleAtLocation(baseLocalisationLocation) : nil

# Check that for lproj folder that contains a language id designator and/or region designator, that there is also a generic lproj

print "Checking #{bundleName} has a lowest common denominator lproj folder for each language it supports...\n"
genericLanguageToLprojNamesMap = NSMutableDictionary.dictionary
localisationFolders.each do |currentLocalisationFolder|
	localisationName = currentLocalisationFolder.lastPathComponent.stringByDeletingPathExtension
	allComponents = NSMutableArray.arrayWithObject localisationName
	
	["-", "_"].each do |currentSeparator|
		allComponents.map! do |currentComponent|
			currentComponent.componentsSeparatedByString(currentSeparator)
		end 
		allComponents.flatten!
	end
	
	genericLanguage = allComponents[0]
	
	lprojNamesForGenericLanguage = genericLanguageToLprojNamesMap.objectForKey genericLanguage
	if lprojNamesForGenericLanguage.nil?
		lprojNamesForGenericLanguage = NSMutableArray.array
		genericLanguageToLprojNamesMap.setObject_forKey lprojNamesForGenericLanguage, genericLanguage
	end
	
	lprojNamesForGenericLanguage << allComponents
end
genericLanguageToLprojNamesMap.each do |currentGenericLanguage, currentLocalisationCombinations|
	hasOneMemberLocalisationCombination = currentLocalisationCombinations.select do |currentCombination|
		currentCombination.size == 1
	end
	next if hasOneMemberLocalisationCombination.size > 0
	
	fail "#{currentGenericLanguage} doesn't have a lowest common denominator lproj folder, we expected to find a folder named \"#{currentGenericLanguage}.lproj\""
end

# Gather localisable resources

print "\n"

localisableResourceFileNames = _resourcesFileNamesInBundleAtLocation(developmentRegionLocation)

print "Found localisable resources:\n"
puts localisableResourceFileNames
print "\n"

print "Found NIB files named:\n"
nibFileNames = localisableResourceFileNames.pathsMatchingExtensions(NSArray.arrayWithObject("nib"))
puts nibFileNames
print "\n"

print "Found string tables named:\n"
stringTableFileNames = localisableResourceFileNames.pathsMatchingExtensions(NSArray.arrayWithObject("strings"))
puts stringTableFileNames
print "\n"


# Validate each region, compare foreign regions to the development region

checkLocalisationFolder = lambda do |currentLocalisationFolder|
	next if currentLocalisationFolder.fileReferenceURL.lastPathComponent.isEqual_ "Base.lproj".to_ns
	
	print "Validating contents of localisation folder #{currentLocalisationFolder.lastPathComponent}...\n"
	
	lprojCheckLocation = NSURL.fileURLWithPath(libraryDirectoryPath.to_ns).URLByAppendingPathComponent("lproj_check.rb")
	
	lprojCheckArguments = NSMutableArray.array
	lprojCheckArguments << "-lprojPath"
	lprojCheckArguments << currentLocalisationFolder.path
	
	$stdout.flush
	lprojCheckTask = NSTask.launchedTaskWithLaunchPath_arguments lprojCheckLocation.path, lprojCheckArguments
	lprojCheckTask.waitUntilExit
	
	if lprojCheckTask.terminationStatus == 0
		print "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are valid\n"
	else
		fail "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are invalid"
	end
	
	
	next if currentLocalisationFolder.fileReferenceURL.isEqual_ developmentRegionLocation.fileReferenceURL
	
	print "Comparing contents of localisation folder #{currentLocalisationFolder.lastPathComponent} to the development region...\n"
	
	contentsPresentAndCorrect = true
	localisableResourceFileNames.each do |currentLocalisableResourceFileName|
		currentResourceLocation = currentLocalisationFolder.URLByAppendingPathComponent currentLocalisableResourceFileName
		next if NSFileManager.defaultManager.fileExistsAtPath currentResourceLocation.path
		
		fail "#{currentLocalisationFolder.lastPathComponent} is missing resource named #{currentLocalisableResourceFileName}"
		contentsPresentAndCorrect = false
	end
	if contentsPresentAndCorrect
		print "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are present and correct\n"
	else
		fail "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are absent"
	end
	unless currentLocalisationFolder.isEqual_ localisationFolders.lastObject
		print "\n"
	end
end

# validate Base localisation if present

compareToBaseLocalisation = lambda do |currentLocalisationFolder|
	print "Validating contents of localisation folder #{currentLocalisationFolder.lastPathComponent} against the Base localisation...\n"
	
	contentsPresentAndCorrect = true
	baseLocalisableResourceFileNames.each do |currentLocalisableResourceFileName|
		currentResourceLocation = currentLocalisationFolder.URLByAppendingPathComponent currentLocalisableResourceFileName
		extensionLessCurrentResourceLocation = currentResourceLocation.URLByDeletingPathExtension
		
		stringCurrentResourceLocation = extensionLessCurrentResourceLocation.URLByAppendingPathExtension "strings"
		next if NSFileManager.defaultManager.fileExistsAtPath stringCurrentResourceLocation.path
		
		fail "#{currentLocalisationFolder.lastPathComponent} is missing resource named #{currentLocalisableResourceFileName} from the Base localisation"
		contentsPresentAndCorrect = false
	end
	
	if contentsPresentAndCorrect
		print "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are matching the Base localisation\n"
	else
		fail "Contents of localisation folder #{currentLocalisationFolder.lastPathComponent} are not matching the Base localisation"
	end
end

localisationFolders.each do |currentLocalisationFolder|
	getSymlink = currentLocalisationFolder.getResourceValue_forKey_error NSURLIsSymbolicLinkKey
	if (getSymlink[0].to_ns.isEqual true.to_ns) && (getSymlink[1].isEqual true.to_ns)
		next
	end
	
	checkLocalisationFolder.call(currentLocalisationFolder)
	
	compareToBaseLocalisation.call(currentLocalisationFolder) unless currentLocalisationFolder.fileReferenceURL.lastPathComponent.isEqual_("Base.lproj") || baseLocalisationLocation.nil?
end

#

finish
